#include <LiquidCrystal.h>
#include <TimerOne.h>

// simple sketch to display a digital clock on an LCD
// see the LiquidCrystal documentation for more info on this
// http://hacks2try.blogspot.com/2010/10/date-and-time-on-arduino-lcd-shield.html

// Version 3 mit Timer
// http://www.arduino.cc/playground/Deutsch/HalloWeltMitInterruptUndTimerlibrary

// buttons
// http://www.dfrobot.com/wiki/index.php?title=Arduino_LCD%26KeyPad_Shield_(SKU:_DFR0009)

int second=0, minute=0, hour=0, weekday=1; // declare time variables
// these time variables are declared globally so they can be used ANYWHERE in your program

LiquidCrystal lcd(8, 9, 4, 5, 6, 7);  // D-Robotics LCD keypad shield
int backLight = 10; // pin will control the backlight

// define some values used by the panel and buttons 
int lcd_key     = 0; 
int adc_key_in  = 0; 
#define btnRIGHT  0 
#define btnUP     1 
#define btnDOWN   2 
#define btnLEFT   3 
#define btnSELECT 4 
#define btnNONE   5   

// read the buttons 
int read_LCD_buttons() 
{  
  adc_key_in = analogRead(0);      // read the value from the sensor   
  // my buttons when read are centered at these valies: 0, 144, 329, 504, 741  
  // we add approx 50 to those values and check to see if we are close  
  if (adc_key_in > 1000) return btnNONE; // We make this the 1st option for speed reasons since it will be the most likely result  
  if (adc_key_in < 50)   return btnRIGHT;    
  if (adc_key_in < 195)  return btnUP;   
  if (adc_key_in < 380)  return btnDOWN;  
  if (adc_key_in < 555)  return btnLEFT;   
  if (adc_key_in < 790)  return btnSELECT;    
  return btnNONE;  // when all others fail, return this... 
}   


void setup(){
  //pinMode(backLight, OUTPUT);
  //digitalWrite(backLight, HIGH); // turn backlight on. Replace 'HIGH' with 'LOW' to turn it off.

  lcd.clear();
  lcd.begin(16,2);

  // test: current time
  hour=16;
  minute=00;
  weekday=5;

  // Timer init.
  Timer1.initialize(1000000); // 1 Sekunde
  Timer1.attachInterrupt(nextSecond); // auszuführende Routine
}



void loop() {
  // set up a local variable to hold the last time we moved forward one second
  // (static variables are initialized once and keep their values between function calls)
  static int ss = 0;

  if (ss != second) {
    ss = second;
    printTime();
    printWeekday(weekday);
  }
  checkButtons(); // runs a function that checks the setting buttons

  delay(200);  // wait some time
}

void nextSecond() {
  second++;

  // move forward one minute every 60 seconds
  if (second > 59) {
    minute++;
    second = 0; // reset seconds to zero

    // move forward one hour every 60 minutes
    if (minute > 59) {
      hour++;
      minute = 0; // reset minutes to zero

      // move forward one weekday every 24 hours
      if (hour > 23) {
        weekday++;
        hour = 0; // reset hours to zero

        // reset weekdays on Saturday
        if (weekday > 7) {
          weekday = 1;
        }
      }
    }
  }
}


void printDigits(byte digits){
  // utility function for digital clock display: prints preceding colon and leading 0
  lcd.print(":");
  if(digits < 10)
    lcd.print('0');
  lcd.print(digits,DEC);
}

void printTime(){
  //  lcd.clear();
  //  lcd.begin(16,2);
  lcd.setCursor(4,1);

  if(hour <10)
    lcd.print(" ");
  // digital clock display of current time
  lcd.print(hour,DEC);

  printDigits(minute);
  printDigits(second);
}

void printWeekday (int dayNum) {
  // print a weekday, based on the day number
  lcd.setCursor(6,0);
  switch (dayNum) {
  case 1:  
    lcd.print ("Sunday   ");  
    break;
  case 2:  
    lcd.print ("Monday   ");  
    break;
  case 3:  
    lcd.print ("Tuesday  ");  
    break;
  case 4:  
    lcd.print ("Wednesday");  
    break;
  case 5:  
    lcd.print ("Thursday ");  
    break;
  case 6:  
    lcd.print ("Friday   ");  
    break;
  case 7:  
    lcd.print ("Saturday ");  
    break;
  }
}

void checkButtons() {
  lcd_key = read_LCD_buttons();  // read the buttons 
  switch (lcd_key)               // depending on which button was pushed, we perform an action  
  { 
  case btnNONE:
    {
      return;
    }

  case btnRIGHT:      
    {      
      minute++;
      if (minute > 59) {
        minute = 0; // reset hours to zero
      }
      printTime();
      break;      
    }   
  case btnLEFT:      
    {      
      hour++;
      if (hour > 23) {
        hour = 0; // reset hours to zero
      }
      printTime();
      break;      
    }   
  case btnSELECT:      
    {      
      weekday++;
      if (weekday > 7) {
        weekday = 1;
      }
      printWeekday(weekday);
      break;      
    }   
  case btnDOWN:      
    {      
      second = 0;
      printTime();
      break;      
    }   
  }
}

