#include <LiquidCrystal.h>
#include <TimerOne.h>

// simple sketch to display a digital clock on an LCD
// see the LiquidCrystal documentation for more info on this
// http://hacks2try.blogspot.com/2010/10/date-and-time-on-arduino-lcd-shield.html

// Version 3 mit Timer
// http://www.arduino.cc/playground/Deutsch/HalloWeltMitInterruptUndTimerlibrary

// buttons
// http://www.dfrobot.com/wiki/index.php?title=Arduino_LCD%26KeyPad_Shield_(SKU:_DFR0009)

int second=0, minute=0, hour=0, weekday=1; // declare time variables
// these time variables are declared globally so they can be used ANYWHERE in your program

LiquidCrystal lcd(8, 9, 4, 5, 6, 7);  // D-Robotics LCD keypad shield
int backLight = 10; // pin will control the backlight

// define some values used by the panel and buttons 
int lcd_key     = 0; 
int adc_key_in  = 0; 
#define btnRIGHT  0 
#define btnUP     1 
#define btnDOWN   2 
#define btnLEFT   3 
#define btnSELECT 4 
#define btnNONE   5   

//*******************************************
//* big numbers
//*******************************************
// based on http://www.arduino.cc/cgi-bin/yabb2/YaBB.pl?num=1265696343

/*
A set of custom made large numbers for a 16x2 LCD using the
 LiquidCrystal librabry. Works with displays compatible with the
 Hitachi HD44780 driver.
 
 Made by Michael Pilcher  2/9/2010
 */

// the 8 arrays that form each segment of the custom numbers
byte C0[8] =
{
  B11111,
  B11111,
  B11111,
  B00000,
  B00000,
  B00000,
  B00000,
  B00000
};
byte C1[8] =
{
  B00000,
  B00000,
  B00000,
  B00000,
  B00000,
  B11111,
  B11111,
  B11111
};
byte C2[8] =
{
  B00000,
  B00000,
  B00000,
  B00000,
  B00000,
  B11100,
  B11100,
  B11100
};
byte C3[8] =
{
  B11111,
  B11111,
  B11111,
  B00000,
  B00000,
  B00000,
  B11111,
  B11111
};
byte C4[8] =
{
  B11100,
  B11100,
  B11100,
  B00000,
  B00000,
  B00000,
  B00000,
  B00000
};
byte C5[8] =
{
  B00011,
  B00011,
  B00011,
  B00011,
  B00011,
  B11111,
  B11111,
  B11111
};
byte C6[8] =
{
  B11100,
  B11100,
  B11100,
  B11100,
  B11100,
  B11100,
  B11100,
  B11100
};
byte C7[8] =
{
  B11100,
  B11100,
  B11100,
  B00000,
  B00000,
  B00000,
  B11100,
  B11100
};

void custom1(int x)
{ // uses segments to build the number 0
  lcd.setCursor(x, 0); // set cursor to column 0, line 0 (first row)
  lcd.write(0);  // call each segment to create
  lcd.write(255);  // top half of the number
  lcd.write(32);
  lcd.setCursor(x, 1); // set cursor to colum 0, line 1 (second row)
  lcd.write(1);  // call each segment to create
  lcd.write(255);  // bottom half of the number
  lcd.write(2);
}

void custom2(int x)
{
  lcd.setCursor(x,0);
  lcd.write(0);
  lcd.write(3);
  lcd.write(6);
  lcd.setCursor(x, 1);
  lcd.write(255);
  lcd.write(1);
  lcd.write(2);
}

void custom3(int x)
{
  lcd.setCursor(x,0);
  lcd.write(3);
  lcd.write(3);
  lcd.write(6);
  lcd.setCursor(x, 1);
  lcd.write(1);
  lcd.write(5);
  lcd.write(6);
}

void custom4(int x)
{
  lcd.setCursor(x,0);
  lcd.write(255);
  lcd.write(1);
  lcd.write(2);
  lcd.setCursor(x, 1);
  lcd.write(32);
  lcd.write(255);
  lcd.write(32);
}

void custom5(int x)
{
  lcd.setCursor(x,0);
  lcd.write(255);
  lcd.write(3);
  lcd.write(7);
  lcd.setCursor(x, 1);
  lcd.write(1);
  lcd.write(5);
  lcd.write(6);
}

void custom6(int x)
{
  lcd.setCursor(x,0);
  lcd.write(255);
  lcd.write(3);
  lcd.write(7);
  lcd.setCursor(x, 1);
  lcd.write(255);
  lcd.write(5);
  lcd.write(6);
}

void custom7(int x)
{
  lcd.setCursor(x,0);
  lcd.write(0);
  lcd.write(0);
  lcd.write(6);
  lcd.setCursor(x, 1);
  lcd.write(32);
  lcd.write(255);
  lcd.write(32);
}

void custom8(int x)
{
  lcd.setCursor(x,0);
  lcd.write(255);
  lcd.write(3);
  lcd.write(6);
  lcd.setCursor(x, 1);
  lcd.write(255);
  lcd.write(5);
  lcd.write(6);
}

void custom9(int x)
{
  lcd.setCursor(x,0);
  lcd.write(255);
  lcd.write(3);
  lcd.write(6);
  lcd.setCursor(x, 1);
  lcd.write(1);
  lcd.write(5);
  lcd.write(6);
}

void custom0(int x)
{
  lcd.setCursor(x,0);
  lcd.write(255);
  lcd.write(0);
  lcd.write(6);
  lcd.setCursor(x, 1);
  lcd.write(255);
  lcd.write(5);
  lcd.write(6);
}

void bignum(int digit, int x) 
{
  switch (digit)               // depending on which button was pushed, we perform an action  
  { 
  case 0: 
    { 
      custom0(x); 
      break; 
    }
  case 1: 
    { 
      custom1(x); 
      break; 
    }
  case 2: 
    { 
      custom2(x); 
      break; 
    }
  case 3: 
    { 
      custom3(x); 
      break; 
    }
  case 4: 
    { 
      custom4(x); 
      break; 
    }
  case 5: 
    { 
      custom5(x); 
      break; 
    }
  case 6: 
    { 
      custom6(x); 
      break; 
    }
  case 7: 
    { 
      custom7(x); 
      break; 
    }
  case 8: 
    { 
      custom8(x); 
      break; 
    }
  case 9: 
    { 
      custom9(x); 
      break; 
    }
  }
}


// read the buttons 
int read_LCD_buttons() 
{  
  adc_key_in = analogRead(0);      // read the value from the sensor   
  // my buttons when read are centered at these valies: 0, 144, 329, 504, 741  
  // we add approx 50 to those values and check to see if we are close  
  if (adc_key_in > 1000) return btnNONE; // We make this the 1st option for speed reasons since it will be the most likely result  
  if (adc_key_in < 50)   return btnRIGHT;    
  if (adc_key_in < 195)  return btnUP;   
  if (adc_key_in < 380)  return btnDOWN;  
  if (adc_key_in < 555)  return btnLEFT;   
  if (adc_key_in < 790)  return btnSELECT;    
  return btnNONE;  // when all others fail, return this... 
}   


void setup(){
  //pinMode(backLight, OUTPUT);
  //digitalWrite(backLight, HIGH); // turn backlight on. Replace 'HIGH' with 'LOW' to turn it off.

  lcd.clear();
  lcd.begin(16,2);

  // test: current time
  hour=16;
  minute=00;
  weekday=5;

  // Timer init.
  Timer1.initialize(1000000); // 1 Sekunde
  Timer1.attachInterrupt(nextSecond); // auszuführende Routine


  // assignes each segment a write number
  lcd.createChar(0,C0);
  lcd.createChar(1,C1);
  lcd.createChar(2,C2);
  lcd.createChar(3,C3);
  lcd.createChar(4,C4);
  lcd.createChar(5,C5);
  lcd.createChar(6,C6);
  lcd.createChar(7,C7);
}



void loop() {
  // set up a local variable to hold the last time we moved forward one second
  // (static variables are initialized once and keep their values between function calls)
  static int ss = 0;

  if (ss != second) {
    ss = second;
    printTime();
    printWeekday(weekday);
  }
  checkButtons(); // runs a function that checks the setting buttons

    delay(200);  // wait some time
}

void nextSecond() {
  second++;

  // move forward one minute every 60 seconds
  if (second > 59) {
    minute++;
    second = 0; // reset seconds to zero

    // move forward one hour every 60 minutes
    if (minute > 59) {
      hour++;
      minute = 0; // reset minutes to zero

      // move forward one weekday every 24 hours
      if (hour > 23) {
        weekday++;
        hour = 0; // reset hours to zero

        // reset weekdays on Saturday
        if (weekday > 7) {
          weekday = 1;
        }
      }
    }
  }
}

void printDigits(byte digits){
  // utility function for digital clock display: prints preceding colon and leading 0
  if(digits < 10)
    lcd.print('0');
  lcd.print(digits,DEC);
}

void printBigDigits(byte digits, int x){
  // utility function for digital clock display: prints preceding colon and leading 0
  bignum(digits/10, x);
  bignum(digits%10, x+3);
}

void printTime(){
  lcd.setCursor(4,1);
  printBigDigits(hour, 0);
  //  lcd.print(":");
  printBigDigits(minute, 7);
  //  lcd.print(":");
  lcd.setCursor(14,1);
  printDigits(second);
}

void printWeekday (int dayNum) {
  // print a weekday, based on the day number
  lcd.setCursor(14,0);
  switch (dayNum) {
  case 1:  
    lcd.print ("Su");  
    break;
  case 2:  
    lcd.print ("Mo");  
    break;
  case 3:  
    lcd.print ("Di");  
    break;
  case 4:  
    lcd.print ("Mi");  
    break;
  case 5:  
    lcd.print ("Do");  
    break;
  case 6:  
    lcd.print ("Fr");  
    break;
  case 7:  
    lcd.print ("Sa");  
    break;
  }
}

void checkButtons() {
  lcd_key = read_LCD_buttons();  // read the buttons 
  switch (lcd_key)               // depending on which button was pushed, we perform an action  
  { 
  case btnNONE:
    {
      return;
    }

  case btnRIGHT:      
    {      
      minute++;
      if (minute > 59) {
        minute = 0; // reset hours to zero
      }
      printTime();
      break;      
    }   
  case btnLEFT:      
    {      
      hour++;
      if (hour > 23) {
        hour = 0; // reset hours to zero
      }
      printTime();
      break;      
    }   
  case btnSELECT:      
    {      
      weekday++;
      if (weekday > 7) {
        weekday = 1;
      }
      printWeekday(weekday);
      break;      
    }   
  case btnDOWN:      
    {      
      second = 0;
      printTime();
      break;      
    }   
  }
}


